@@ -559,6 +559,12 @@ static int __try_to_segment(struct sk_buff *skb, bool csum_partial,
 
 static int try_to_segment(struct sk_buff *skb)
 {
+#ifdef SKIP_ZERO_COPY
+	/* coalesce_skb() since does not generate frag-list no need to
+	 * linearize it here.
+	 */
+	return 0;
+#else
 	struct stthdr *stth = stt_hdr(skb);
 	bool csum_partial = !!(stth->flags & STT_CSUM_PARTIAL);
 	bool ipv4 = !!(stth->flags & STT_PROTO_IPV4);
@@ -566,6 +572,7 @@ static int try_to_segment(struct sk_buff *skb)
 	int l4_offset = stth->l4_offset;
 
 	return __try_to_segment(skb, csum_partial, ipv4, tcp, l4_offset);
+#endif
 }
 
 static int segment_skb(struct sk_buff **headp, bool csum_partial,
@@ -1027,7 +1034,7 @@ netdev_tx_t ovs_stt_xmit(struct sk_buff *skb)
 error:
 	kfree_skb(skb);
 	dev->stats.tx_errors++;
-	return err;
+	return NETDEV_TX_OK;
 }
 EXPORT_SYMBOL(ovs_stt_xmit);
 
