@@ -361,14 +361,11 @@ monitor_daemon(pid_t daemon_pid)
                                (unsigned long int) daemon_pid, status_msg);
 
         if (child_ready) {
-            int error;
             do {
                 retval = waitpid(daemon_pid, &status, 0);
-                error = retval == -1 ? errno : 0;
-            } while (error == EINTR);
-            vlog_reopen_log_file();
-            if (error) {
-                VLOG_FATAL("waitpid failed (%s)", ovs_strerror(error));
+            } while (retval == -1 && errno == EINTR);
+            if (retval == -1) {
+                VLOG_FATAL("waitpid failed (%s)", ovs_strerror(errno));
             }
         }
 
@@ -428,7 +425,7 @@ monitor_daemon(pid_t daemon_pid)
 
     /* Running in new daemon process. */
     ovs_cmdl_proctitle_restore();
-    set_subprogram_name(program_name);
+    set_subprogram_name("");
 }
 
 /* If daemonization is configured, then starts daemonization, by forking and
